/*
 * =====================================================================================
 *
 *       Filename:  HistoNames.h
 *
 *    Description:  Histogram names for Global DQM 
 *
 *        Version:  2.0
 *        Created:  10/03/2008 11:54:31 AM
 *       Revision:  none
 *       Compiler:  gcc
 *
 *         Author:  Valdas Rapsevicius (VR), Valdas.Rapsevicius@cern.ch
 *        Company:  CERN, CH
 *
 * =====================================================================================
 */

#ifndef CSCDQM_HistoNames_H
#define CSCDQM_HistoNames_H 


namespace h {

/**
 *  List of histogram identifiers and names. Please add or update list items in
 *  a form:
 *  HIST_ID(ID)
 *  , where ID should be unique with prefix CSC, DDU, EMU or PAR, name should
 *  correlate with histogram name from XML Booking file or just to be unique in
 *  type.
 */

#define HIST_ID_TABLE \
  HIST_ID(CSC_ACTUAL_DMB_CFEB_DAV_FREQUENCY) \
  HIST_ID(CSC_ACTUAL_DMB_CFEB_DAV_MULTIPLICITY_FREQUENCY) \
  HIST_ID(CSC_ACTUAL_DMB_CFEB_DAV_MULTIPLICITY_RATE) \
  HIST_ID(CSC_ACTUAL_DMB_CFEB_DAV_RATE) \
  HIST_ID(CSC_ACTUAL_DMB_FEB_COMBINATIONS_DAV_FREQUENCY) \
  HIST_ID(CSC_ACTUAL_DMB_FEB_COMBINATIONS_DAV_RATE) \
  HIST_ID(CSC_ACTUAL_DMB_FEB_DAV_FREQUENCY) \
  HIST_ID(CSC_ACTUAL_DMB_FEB_DAV_RATE) \
  HIST_ID(CSC_ALCT1_VS_ALCT0_KEYWG) \
  HIST_ID(CSC_ALCTTIME_LYXX) \
  HIST_ID(CSC_ALCTTIME_LYXX_PROFILE) \
  HIST_ID(CSC_ALCTXX_BXN) \
  HIST_ID(CSC_ALCTXX_DTIME) \
  HIST_ID(CSC_ALCTXX_DTIME_PROFILE) \
  HIST_ID(CSC_ALCTXX_DTIME_VS_KEYWG) \
  HIST_ID(CSC_ALCTXX_KEYWG) \
  HIST_ID(CSC_ALCTXX_PATTERN) \
  HIST_ID(CSC_ALCTXX_PATTERN_DISTR) \
  HIST_ID(CSC_ALCTXX_QUALITY) \
  HIST_ID(CSC_ALCTXX_QUALITY_DISTR) \
  HIST_ID(CSC_ALCTXX_QUALITY_PROFILE) \
  HIST_ID(CSC_ALCT_BXN) \
  HIST_ID(CSC_ALCT_BXN_VS_DMB_BXN) \
  HIST_ID(CSC_ALCT_DMB_BXN_DIFF) \
  HIST_ID(CSC_ALCT_DMB_L1A_DIFF) \
  HIST_ID(CSC_ALCT_L1A) \
  HIST_ID(CSC_ALCT_LYXX_EFFICIENCY) \
  HIST_ID(CSC_ALCT_LYXX_RATE) \
  HIST_ID(CSC_ALCT_MATCH_TIME) \
  HIST_ID(CSC_ALCT_NUMBER_EFFICIENCY) \
  HIST_ID(CSC_ALCT_NUMBER_OF_LAYERS_WITH_HITS) \
  HIST_ID(CSC_ALCT_NUMBER_OF_WIREGROUPS_WITH_HITS) \
  HIST_ID(CSC_ALCT_NUMBER_RATE) \
  HIST_ID(CSC_ALCT_WORD_COUNT) \
  HIST_ID(CSC_BINCHECK_DATAFLOW_PROBLEMS_FREQUENCY) \
  HIST_ID(CSC_BINCHECK_DATAFLOW_PROBLEMS_TABLE) \
  HIST_ID(CSC_BINCHECK_ERRORSTAT_TABLE) \
  HIST_ID(CSC_BINCHECK_ERRORS_FREQUENCY) \
  HIST_ID(CSC_EVENT_DISPLAY_NOXX) \
  HIST_ID(CSC_CFEBXX_DMB_L1A_DIFF) \
  HIST_ID(CSC_CFEBXX_FREE_SCA_CELLS) \
  HIST_ID(CSC_CFEBXX_L1A_SYNC_TIME) \
  HIST_ID(CSC_CFEBXX_L1A_SYNC_TIME_DMB_DIFF) \
  HIST_ID(CSC_CFEBXX_L1A_SYNC_TIME_VS_DMB) \
  HIST_ID(CSC_CFEBXX_LCT_PHASE_VS_L1A_PHASE) \
  HIST_ID(CSC_CFEBXX_SCA_BLOCKS_LOCKED_BY_LCTS) \
  HIST_ID(CSC_CFEBXX_SCA_BLOCKS_LOCKED_BY_LCTXL1) \
  HIST_ID(CSC_CFEBXX_SCA_BLOCK_OCCUPANCY) \
  HIST_ID(CSC_CFEB_ACTIVESTRIPS_LYXX) \
  HIST_ID(CSC_CFEB_ACTIVE_SAMPLES_VS_STRIP_LYXX) \
  HIST_ID(CSC_CFEB_ACTIVE_SAMPLES_VS_STRIP_LYXX_PROFILE) \
  HIST_ID(CSC_CFEB_AFEB_RAWHITS_TIMEBINS) \
  HIST_ID(CSC_CFEB_CLUSTERS_CHARGE_LY_XX) \
  HIST_ID(CSC_CFEB_CLUSTER_DURATION_LY_XX) \
  HIST_ID(CSC_CFEB_NUMBER_OF_CLUSTERS_LY_XX) \
  HIST_ID(CSC_CFEB_OUT_OFF_RANGE_STRIPS_LYXX) \
  HIST_ID(CSC_CFEB_PEDESTALRMS_SAMPLE_01_LYXX) \
  HIST_ID(CSC_CFEB_PEDESTAL_WITHEMV_SAMPLE_01_LYXX) \
  HIST_ID(CSC_CFEB_PEDESTAL_WITHRMS_SAMPLE_01_LYXX) \
  HIST_ID(CSC_CFEB_PEDESTAL__WITHEMV__SAMPLE_01_LYXX) \
  HIST_ID(CSC_CFEB_PEDESTAL__WITHRMS__SAMPLE_01_LYXX) \
  HIST_ID(CSC_CFEB_SCA_CELL_PEAK_LY_XX) \
  HIST_ID(CSC_CFEB_WIDTH_OF_CLUSTERS_LY_XX) \
  HIST_ID(CSC_CLCT0_CLCT1_CLSSIFICATION) \
  HIST_ID(CSC_CLCT0_CLSSIFICATION) \
  HIST_ID(CSC_CLCT0_KEYDISTRIP_VS_ALCT0_KEYWIREGROUP) \
  HIST_ID(CSC_CLCT1_VS_CLCT0_KEY_STRIP) \
  HIST_ID(CSC_CLCTTIME_LYXX) \
  HIST_ID(CSC_CLCTTIME_LYXX_PROFILE) \
  HIST_ID(CSC_CLCTXX_BXN) \
  HIST_ID(CSC_CLCTXX_DISTRIP_PATTERN) \
  HIST_ID(CSC_CLCTXX_DISTRIP_QUALITY) \
  HIST_ID(CSC_CLCTXX_DISTRIP_QUALITY_PROFILE) \
  HIST_ID(CSC_CLCTXX_DTIME) \
  HIST_ID(CSC_CLCTXX_DTIME_PROFILE) \
  HIST_ID(CSC_CLCTXX_DTIME_VS_DISTRIP) \
  HIST_ID(CSC_CLCTXX_DTIME_VS_HALF_STRIP) \
  HIST_ID(CSC_CLCTXX_HALF_STRIP_PATTERN) \
  HIST_ID(CSC_CLCTXX_HALF_STRIP_QUALITY) \
  HIST_ID(CSC_CLCTXX_HALF_STRIP_QUALITY_DISTR) \
  HIST_ID(CSC_CLCTXX_HALF_STRIP_QUALITY_PROFILE) \
  HIST_ID(CSC_CLCTXX_KEYDISTRIP) \
  HIST_ID(CSC_CLCTXX_KEYHALFSTRIP) \
  HIST_ID(CSC_CLCT_BXN) \
  HIST_ID(CSC_CLCT_BXN_VS_DMB_BXN) \
  HIST_ID(CSC_CLCT_DMB_BXN_DIFF) \
  HIST_ID(CSC_CLCT_DMB_L1A_DIFF) \
  HIST_ID(CSC_CLCT_HALF_STRIP_PATTERN_DISTR) \
  HIST_ID(CSC_CLCT_L1A) \
  HIST_ID(CSC_CLCT_LYXX_EFFICIENCY) \
  HIST_ID(CSC_CLCT_LYXX_RATE) \
  HIST_ID(CSC_CLCT_NUMBER) \
  HIST_ID(CSC_CLCT_NUMBER_OF_HALFSTRIPS_WITH_HITS) \
  HIST_ID(CSC_CLCT_NUMBER_OF_LAYERS_WITH_HITS) \
  HIST_ID(CSC_CLCT_NUMBER_RATE) \
  HIST_ID(CSC_CSC_EFFICIENCY) \
  HIST_ID(CSC_CSC_RATE) \
  HIST_ID(CSC_DMB_BXN_DISTRIB) \
  HIST_ID(CSC_DMB_BXN_VS_DDU_BXN) \
  HIST_ID(CSC_DMB_CFEB_ACTIVE) \
  HIST_ID(CSC_DMB_CFEB_ACTIVE_VS_DAV) \
  HIST_ID(CSC_DMB_CFEB_DAV) \
  HIST_ID(CSC_DMB_CFEB_DAV_MULTIPLICITY) \
  HIST_ID(CSC_DMB_CFEB_DAV_MULTIPLICITY_UNPACKING_INEFFICIENCY) \
  HIST_ID(CSC_DMB_CFEB_DAV_UNPACKING_INEFFICIENCY) \
  HIST_ID(CSC_DMB_CFEB_MOVLP) \
  HIST_ID(CSC_DMB_CFEB_SYNC) \
  HIST_ID(CSC_DMB_DDU_BXN_DIFF) \
  HIST_ID(CSC_DMB_DDU_L1A_DIFF) \
  HIST_ID(CSC_DMB_FEB_COMBINATIONS_DAV_EFFICIENCY) \
  HIST_ID(CSC_DMB_FEB_COMBINATIONS_DAV_RATE) \
  HIST_ID(CSC_DMB_FEB_COMBINATIONS_DAV_UNPACKING_INEFFICIENCY) \
  HIST_ID(CSC_DMB_FEB_COMBINATIONS_UNPACKED_VS_DAV) \
  HIST_ID(CSC_DMB_FEB_DAV_EFFICIENCY) \
  HIST_ID(CSC_DMB_FEB_DAV_RATE) \
  HIST_ID(CSC_DMB_FEB_DAV_UNPACKING_INEFFICIENCY) \
  HIST_ID(CSC_DMB_FEB_TIMEOUTS) \
  HIST_ID(CSC_DMB_FEB_UNPACKED_VS_DAV) \
  HIST_ID(CSC_DMB_FIFO_STATS) \
  HIST_ID(CSC_DMB_L1A_DISTRIB) \
  HIST_ID(CSC_DMB_L1A_VS_ALCT_L1A) \
  HIST_ID(CSC_DMB_L1A_VS_CLCT_L1A) \
  HIST_ID(CSC_DMB_L1A_VS_DDU_L1A) \
  HIST_ID(CSC_DMB_L1_PIPE) \
  HIST_ID(CSC_LCT0_MATCH_BXN_DIFFERENCE) \
  HIST_ID(CSC_LCT1_MATCH_BXN_DIFFERENCE) \
  HIST_ID(CSC_LCT_MATCH_STATUS) \
  HIST_ID(CSC_TMB_ALCT_BXN_DIFF) \
  HIST_ID(CSC_TMB_ALCT_L1A_DIFF) \
  HIST_ID(CSC_TMB_BXN_VS_ALCT_BXN) \
  HIST_ID(CSC_TMB_L1A_VS_ALCT_L1A) \
  HIST_ID(CSC_TMB_WORD_COUNT) \
  HIST_ID(CSC_CFEB_COMPARATORS_TIMESAMPLES) \
  HIST_ID(DDU_BUFFER_SIZE) \
  HIST_ID(DDU_BXN) \
  HIST_ID(DDU_CSC_ERRORS) \
  HIST_ID(DDU_CSC_ERRORS_RATE) \
  HIST_ID(DDU_CSC_WARNINGS) \
  HIST_ID(DDU_CSC_WARNINGS_RATE) \
  HIST_ID(DDU_DMB_ACTIVE_HEADER_COUNT) \
  HIST_ID(DDU_DMB_CONNECTED_INPUTS) \
  HIST_ID(DDU_DMB_CONNECTED_INPUTS_RATE) \
  HIST_ID(DDU_DMB_DAV_HEADER_COUNT_VS_DMB_ACTIVE_HEADER_COUNT) \
  HIST_ID(DDU_DMB_DAV_HEADER_OCCUPANCY) \
  HIST_ID(DDU_DMB_DAV_HEADER_OCCUPANCY_RATE) \
  HIST_ID(DDU_DMB_UNPACKED_VS_DAV) \
  HIST_ID(DDU_L1A_INCREMENT) \
  HIST_ID(DDU_READOUT_ERRORS) \
  HIST_ID(DDU_TRAILER_ERRORSTAT_FREQUENCY) \
  HIST_ID(DDU_TRAILER_ERRORSTAT_RATE) \
  HIST_ID(DDU_TRAILER_ERRORSTAT_TABLE) \
  HIST_ID(DDU_WORD_COUNT) \
  HIST_ID(EMU_ALL_DDUS_AVERAGE_EVENT_SIZE) \
  HIST_ID(EMU_ALL_DDUS_AVERAGE_INPUTS_WITH_DATA) \
  HIST_ID(EMU_ALL_DDUS_AVERAGE_LIVE_INPUTS) \
  HIST_ID(EMU_ALL_DDUS_EVENT_SIZE) \
  HIST_ID(EMU_ALL_DDUS_FORMAT_ERRORS) \
  HIST_ID(EMU_ALL_DDUS_INPUTS_ERRORS) \
  HIST_ID(EMU_ALL_DDUS_INPUTS_WARNINGS) \
  HIST_ID(EMU_ALL_DDUS_INPUTS_WITH_DATA) \
  HIST_ID(EMU_ALL_DDUS_IN_READOUT) \
  HIST_ID(EMU_ALL_DDUS_L1A_INCREMENT) \
  HIST_ID(EMU_ALL_DDUS_LIVE_INPUTS) \
  HIST_ID(EMU_ALL_DDUS_TRAILER_ERRORS) \
  HIST_ID(EMU_ALL_READOUT_ERRORS) \
  HIST_ID(EMU_CSC_AFEB_ENDCAP_MINUS_RAWHITS_TIME) \
  HIST_ID(EMU_CSC_AFEB_ENDCAP_PLUS_RAWHITS_TIME) \
  HIST_ID(EMU_CSC_AFEB_RAWHITS_TIME_MEAN) \
  HIST_ID(EMU_CSC_AFEB_RAWHITS_TIME_RMS) \
  HIST_ID(EMU_CSC_ALCT0_BXN_MEAN) \
  HIST_ID(EMU_CSC_ALCT0_BXN_RMS) \
  HIST_ID(EMU_CSC_ALCT0_ENDCAP_MINUS_DTIME) \
  HIST_ID(EMU_CSC_ALCT0_ENDCAP_PLUS_DTIME) \
  HIST_ID(EMU_CSC_ALCT0_QUALITY) \
  HIST_ID(EMU_CSC_ALCT_CLCT_MATCH_MEAN) \
  HIST_ID(EMU_CSC_ALCT_CLCT_MATCH_RMS) \
  HIST_ID(EMU_CSC_ALCT_PLANES_WITH_HITS) \
  HIST_ID(EMU_CSC_CLCT0_BXN_MEAN) \
  HIST_ID(EMU_CSC_CLCT0_BXN_RMS) \
  HIST_ID(EMU_CSC_CLCT0_QUALITY) \
  HIST_ID(EMU_CSC_CLCT_PLANES_WITH_HITS) \
  HIST_ID(EMU_CSC_DMB_INPUT_FIFO_FULL) \
  HIST_ID(EMU_CSC_DMB_INPUT_FIFO_FULL_FRACT) \
  HIST_ID(EMU_CSC_DMB_INPUT_TIMEOUT) \
  HIST_ID(EMU_CSC_DMB_INPUT_TIMEOUT_FRACT) \
  HIST_ID(EMU_CSC_ENDCAP_MINUS_ALCT_CLCT_MATCH_TIME) \
  HIST_ID(EMU_CSC_ENDCAP_MINUS_CLCT0_DTIME) \
  HIST_ID(EMU_CSC_ENDCAP_PLUS_ALCT_CLCT_MATCH_TIME) \
  HIST_ID(EMU_CSC_ENDCAP_PLUS_CLCT0_DTIME) \
  HIST_ID(EMU_CSC_ENDCAP_PLUS_CFEB_COMPARATORS_TIME) \
  HIST_ID(EMU_CSC_ENDCAP_MINUS_CFEB_COMPARATORS_TIME) \
  HIST_ID(EMU_CSC_CFEB_COMPARATORS_TIME_MEAN) \
  HIST_ID(EMU_CSC_CFEB_COMPARATORS_TIME_RMS) \
  HIST_ID(EMU_CSC_FORMAT_ERRORS) \
  HIST_ID(EMU_CSC_FORMAT_ERRORS_FRACT) \
  HIST_ID(EMU_CSC_FORMAT_WARNINGS) \
  HIST_ID(EMU_CSC_FORMAT_WARNINGS_FRACT) \
  HIST_ID(EMU_CSC_L1A_OUT_OF_SYNC) \
  HIST_ID(EMU_CSC_L1A_OUT_OF_SYNC_FRACT) \
  HIST_ID(EMU_CSC_REPORTING) \
  HIST_ID(EMU_CSC_STATS_CFEB_BWORDS) \
  HIST_ID(EMU_CSC_STATS_FIFOFULL_ERR) \
  HIST_ID(EMU_CSC_STATS_FORMAT_ERR) \
  HIST_ID(EMU_CSC_STATS_INPUTTO_ERR) \
  HIST_ID(EMU_CSC_STATS_L1SYNC_ERR) \
  HIST_ID(EMU_CSC_STATS_OCCUPANCY) \
  HIST_ID(EMU_CSC_STATS_SUMMARY) \
  HIST_ID(EMU_CSC_STATS_WO_ALCT) \
  HIST_ID(EMU_CSC_STATS_WO_CFEB) \
  HIST_ID(EMU_CSC_STATS_WO_CLCT) \
  HIST_ID(EMU_CSC_UNPACKED) \
  HIST_ID(CSC_CFEB_SCA_CELLPEAK_TIME) \
  HIST_ID(EMU_CSC_PLUS_ENDCAP_CFEB_SCA_CELLPEAK_TIME) \
  HIST_ID(EMU_CSC_MINUS_ENDCAP_CFEB_SCA_CELLPEAK_TIME) \
  HIST_ID(EMU_CSC_CFEB_SCA_CELLPEAK_TIME_MEAN) \
  HIST_ID(EMU_CSC_CFEB_SCA_CELLPEAK_TIME_RMS) \
  HIST_ID(EMU_CSC_UNPACKED_FRACT) \
  HIST_ID(EMU_CSC_UNPACKED_WITH_ERRORS) \
  HIST_ID(EMU_CSC_WO_ALCT) \
  HIST_ID(EMU_CSC_STANDBY) \
  HIST_ID(EMU_CSC_WO_ALCT_FRACT) \
  HIST_ID(EMU_CSC_WO_CFEB) \
  HIST_ID(EMU_CSC_WO_CFEB_FRACT) \
  HIST_ID(EMU_CSC_WO_CLCT) \
  HIST_ID(EMU_CSC_WO_CLCT_FRACT) \
  HIST_ID(EMU_DDU_BXN) \
  HIST_ID(EMU_DMB_FORMAT_ERRORS) \
  HIST_ID(EMU_DMB_FORMAT_ERRORS_FRACT) \
  HIST_ID(EMU_DMB_FORMAT_WARNINGS) \
  HIST_ID(EMU_DMB_FORMAT_WARNINGS_FRACT) \
  HIST_ID(EMU_DMB_INPUT_FIFO_FULL) \
  HIST_ID(EMU_DMB_INPUT_FIFO_FULL_FRACT) \
  HIST_ID(EMU_DMB_INPUT_TIMEOUT) \
  HIST_ID(EMU_DMB_INPUT_TIMEOUT_FRACT) \
  HIST_ID(EMU_DMB_L1A_OUT_OF_SYNC) \
  HIST_ID(EMU_DMB_L1A_OUT_OF_SYNC_FRACT) \
  HIST_ID(EMU_DMB_REPORTING) \
  HIST_ID(EMU_DMB_UNPACKED) \
  HIST_ID(EMU_DMB_UNPACKED_FRACT) \
  HIST_ID(EMU_DMB_UNPACKED_WITH_ERRORS) \
  HIST_ID(EMU_DMB_WO_ALCT) \
  HIST_ID(EMU_DMB_WO_ALCT_FRACT) \
  HIST_ID(EMU_DMB_WO_CFEB) \
  HIST_ID(EMU_DMB_WO_CFEB_FRACT) \
  HIST_ID(EMU_DMB_WO_CLCT) \
  HIST_ID(EMU_DMB_WO_CLCT_FRACT) \
  HIST_ID(EMU_FED_ENTRIES) \
  HIST_ID(EMU_FED_FATAL) \
  HIST_ID(EMU_FED_FORMAT_FATAL) \
  HIST_ID(EMU_FED_NONFATAL) \
  HIST_ID(EMU_PHYSICS_EMU) \
  HIST_ID(EMU_PHYSICS_ME1) \
  HIST_ID(EMU_PHYSICS_ME2) \
  HIST_ID(EMU_PHYSICS_ME3) \
  HIST_ID(EMU_PHYSICS_ME4) \
  HIST_ID(EMU_EVENT_DISPLAY_ANODE) \
  HIST_ID(EMU_EVENT_DISPLAY_CATHODE) \
  HIST_ID(EMU_EVENT_DISPLAY_XY) \
  HIST_ID(PAR_REPORT_SUMMARY) \
  HIST_ID(PAR_CSC_SIDEMINUS) \
  HIST_ID(PAR_CSC_SIDEMINUS_STATION01) \
  HIST_ID(PAR_CSC_SIDEMINUS_STATION01_RING01) \
  HIST_ID(PAR_CSC_SIDEMINUS_STATION01_RING02) \
  HIST_ID(PAR_CSC_SIDEMINUS_STATION01_RING03) \
  HIST_ID(PAR_CSC_SIDEMINUS_STATION02) \
  HIST_ID(PAR_CSC_SIDEMINUS_STATION02_RING01) \
  HIST_ID(PAR_CSC_SIDEMINUS_STATION02_RING02) \
  HIST_ID(PAR_CSC_SIDEMINUS_STATION03) \
  HIST_ID(PAR_CSC_SIDEMINUS_STATION03_RING01) \
  HIST_ID(PAR_CSC_SIDEMINUS_STATION03_RING02) \
  HIST_ID(PAR_CSC_SIDEMINUS_STATION04) \
  HIST_ID(PAR_CSC_SIDEMINUS_STATION04_RING01) \
  HIST_ID(PAR_CSC_SIDEMINUS_STATION04_RING02) \
  HIST_ID(PAR_CSC_SIDEPLUS) \
  HIST_ID(PAR_CSC_SIDEPLUS_STATION01) \
  HIST_ID(PAR_CSC_SIDEPLUS_STATION01_RING01) \
  HIST_ID(PAR_CSC_SIDEPLUS_STATION01_RING02) \
  HIST_ID(PAR_CSC_SIDEPLUS_STATION01_RING03) \
  HIST_ID(PAR_CSC_SIDEPLUS_STATION02) \
  HIST_ID(PAR_CSC_SIDEPLUS_STATION02_RING01) \
  HIST_ID(PAR_CSC_SIDEPLUS_STATION02_RING02) \
  HIST_ID(PAR_CSC_SIDEPLUS_STATION03) \
  HIST_ID(PAR_CSC_SIDEPLUS_STATION03_RING01) \
  HIST_ID(PAR_CSC_SIDEPLUS_STATION03_RING02) \
  HIST_ID(PAR_CSC_SIDEPLUS_STATION04) \
  HIST_ID(PAR_CSC_SIDEPLUS_STATION04_RING01) \
  HIST_ID(PAR_CSC_SIDEPLUS_STATION04_RING02) \
  HIST_ID(PAR_CRT_SUMMARY) \
  HIST_ID(PAR_CRT_SIDEMINUS) \
  HIST_ID(PAR_CRT_SIDEMINUS_STATION01) \
  HIST_ID(PAR_CRT_SIDEMINUS_STATION01_RING01) \
  HIST_ID(PAR_CRT_SIDEMINUS_STATION01_RING02) \
  HIST_ID(PAR_CRT_SIDEMINUS_STATION01_RING03) \
  HIST_ID(PAR_CRT_SIDEMINUS_STATION02) \
  HIST_ID(PAR_CRT_SIDEMINUS_STATION02_RING01) \
  HIST_ID(PAR_CRT_SIDEMINUS_STATION02_RING02) \
  HIST_ID(PAR_CRT_SIDEMINUS_STATION03) \
  HIST_ID(PAR_CRT_SIDEMINUS_STATION03_RING01) \
  HIST_ID(PAR_CRT_SIDEMINUS_STATION03_RING02) \
  HIST_ID(PAR_CRT_SIDEMINUS_STATION04) \
  HIST_ID(PAR_CRT_SIDEMINUS_STATION04_RING01) \
  HIST_ID(PAR_CRT_SIDEMINUS_STATION04_RING02) \
  HIST_ID(PAR_CRT_SIDEPLUS) \
  HIST_ID(PAR_CRT_SIDEPLUS_STATION01) \
  HIST_ID(PAR_CRT_SIDEPLUS_STATION01_RING01) \
  HIST_ID(PAR_CRT_SIDEPLUS_STATION01_RING02) \
  HIST_ID(PAR_CRT_SIDEPLUS_STATION01_RING03) \
  HIST_ID(PAR_CRT_SIDEPLUS_STATION02) \
  HIST_ID(PAR_CRT_SIDEPLUS_STATION02_RING01) \
  HIST_ID(PAR_CRT_SIDEPLUS_STATION02_RING02) \
  HIST_ID(PAR_CRT_SIDEPLUS_STATION03) \
  HIST_ID(PAR_CRT_SIDEPLUS_STATION03_RING01) \
  HIST_ID(PAR_CRT_SIDEPLUS_STATION03_RING02) \
  HIST_ID(PAR_CRT_SIDEPLUS_STATION04) \
  HIST_ID(PAR_CRT_SIDEPLUS_STATION04_RING01) \
  HIST_ID(PAR_CRT_SIDEPLUS_STATION04_RING02) \
  HIST_ID(PAR_DAQ_SUMMARY) \
  HIST_ID(PAR_DAQ_SIDEMINUS) \
  HIST_ID(PAR_DAQ_SIDEMINUS_STATION01) \
  HIST_ID(PAR_DAQ_SIDEMINUS_STATION01_RING01) \
  HIST_ID(PAR_DAQ_SIDEMINUS_STATION01_RING02) \
  HIST_ID(PAR_DAQ_SIDEMINUS_STATION01_RING03) \
  HIST_ID(PAR_DAQ_SIDEMINUS_STATION02) \
  HIST_ID(PAR_DAQ_SIDEMINUS_STATION02_RING01) \
  HIST_ID(PAR_DAQ_SIDEMINUS_STATION02_RING02) \
  HIST_ID(PAR_DAQ_SIDEMINUS_STATION03) \
  HIST_ID(PAR_DAQ_SIDEMINUS_STATION03_RING01) \
  HIST_ID(PAR_DAQ_SIDEMINUS_STATION03_RING02) \
  HIST_ID(PAR_DAQ_SIDEMINUS_STATION04) \
  HIST_ID(PAR_DAQ_SIDEMINUS_STATION04_RING01) \
  HIST_ID(PAR_DAQ_SIDEMINUS_STATION04_RING02) \
  HIST_ID(PAR_DAQ_SIDEPLUS) \
  HIST_ID(PAR_DAQ_SIDEPLUS_STATION01) \
  HIST_ID(PAR_DAQ_SIDEPLUS_STATION01_RING01) \
  HIST_ID(PAR_DAQ_SIDEPLUS_STATION01_RING02) \
  HIST_ID(PAR_DAQ_SIDEPLUS_STATION01_RING03) \
  HIST_ID(PAR_DAQ_SIDEPLUS_STATION02) \
  HIST_ID(PAR_DAQ_SIDEPLUS_STATION02_RING01) \
  HIST_ID(PAR_DAQ_SIDEPLUS_STATION02_RING02) \
  HIST_ID(PAR_DAQ_SIDEPLUS_STATION03) \
  HIST_ID(PAR_DAQ_SIDEPLUS_STATION03_RING01) \
  HIST_ID(PAR_DAQ_SIDEPLUS_STATION03_RING02) \
  HIST_ID(PAR_DAQ_SIDEPLUS_STATION04) \
  HIST_ID(PAR_DAQ_SIDEPLUS_STATION04_RING01) \
  HIST_ID(PAR_DAQ_SIDEPLUS_STATION04_RING02) \
  HIST_ID(PAR_DCS_SUMMARY) \
  HIST_ID(PAR_DCS_SIDEMINUS) \
  HIST_ID(PAR_DCS_SIDEMINUS_STATION01) \
  HIST_ID(PAR_DCS_SIDEMINUS_STATION01_RING01) \
  HIST_ID(PAR_DCS_SIDEMINUS_STATION01_RING02) \
  HIST_ID(PAR_DCS_SIDEMINUS_STATION01_RING03) \
  HIST_ID(PAR_DCS_SIDEMINUS_STATION02) \
  HIST_ID(PAR_DCS_SIDEMINUS_STATION02_RING01) \
  HIST_ID(PAR_DCS_SIDEMINUS_STATION02_RING02) \
  HIST_ID(PAR_DCS_SIDEMINUS_STATION03) \
  HIST_ID(PAR_DCS_SIDEMINUS_STATION03_RING01) \
  HIST_ID(PAR_DCS_SIDEMINUS_STATION03_RING02) \
  HIST_ID(PAR_DCS_SIDEMINUS_STATION04) \
  HIST_ID(PAR_DCS_SIDEMINUS_STATION04_RING01) \
  HIST_ID(PAR_DCS_SIDEMINUS_STATION04_RING02) \
  HIST_ID(PAR_DCS_SIDEPLUS) \
  HIST_ID(PAR_DCS_SIDEPLUS_STATION01) \
  HIST_ID(PAR_DCS_SIDEPLUS_STATION01_RING01) \
  HIST_ID(PAR_DCS_SIDEPLUS_STATION01_RING02) \
  HIST_ID(PAR_DCS_SIDEPLUS_STATION01_RING03) \
  HIST_ID(PAR_DCS_SIDEPLUS_STATION02) \
  HIST_ID(PAR_DCS_SIDEPLUS_STATION02_RING01) \
  HIST_ID(PAR_DCS_SIDEPLUS_STATION02_RING02) \
  HIST_ID(PAR_DCS_SIDEPLUS_STATION03) \
  HIST_ID(PAR_DCS_SIDEPLUS_STATION03_RING01) \
  HIST_ID(PAR_DCS_SIDEPLUS_STATION03_RING02) \
  HIST_ID(PAR_DCS_SIDEPLUS_STATION04) \
  HIST_ID(PAR_DCS_SIDEPLUS_STATION04_RING01) \
  HIST_ID(PAR_DCS_SIDEPLUS_STATION04_RING02) \
  HIST_ID(EMU_FED_BUFFER_SIZE) \
  HIST_ID(EMU_FED_DDU_L1A_MISMATCH) \
  HIST_ID(EMU_FED_EVENT_SIZE) \
  HIST_ID(EMU_FED_TOTAL_CSC_NUMBER) \
  HIST_ID(EMU_FED_TOTAL_CFEB_NUMBER) \
  HIST_ID(EMU_FED_TOTAL_ALCT_NUMBER) \
  HIST_ID(EMU_FED_TOTAL_TMB_NUMBER) \
  HIST_ID(FED_BUFFER_SIZE) \
  HIST_ID(EMU_FED_FORMAT_ERRORS) \
  HIST_ID(EMU_FED_DDU_L1A_MISMATCH_FRACT) \
  HIST_ID(EMU_FED_DDU_L1A_MISMATCH_WITH_CSC_DATA) \
  HIST_ID(EMU_FED_DDU_L1A_MISMATCH_WITH_CSC_DATA_FRACT) \
  HIST_ID(EMU_FED_DDU_L1A_MISMATCH_CNT) \
  HIST_ID(EMU_FED_DDU_L1A_MISMATCH_WITH_CSC_DATA_CNT) \
  HIST_ID(EMU_FED_STATS)

#define HIST_NAME_TABLE \
  HIST_NAME("Actual_DMB_CFEB_DAV_Frequency") \
  HIST_NAME("Actual_DMB_CFEB_DAV_multiplicity_Frequency") \
  HIST_NAME("Actual_DMB_CFEB_DAV_multiplicity_Rate") \
  HIST_NAME("Actual_DMB_CFEB_DAV_Rate") \
  HIST_NAME("Actual_DMB_FEB_Combinations_DAV_Frequency") \
  HIST_NAME("Actual_DMB_FEB_Combinations_DAV_Rate") \
  HIST_NAME("Actual_DMB_FEB_DAV_Frequency") \
  HIST_NAME("Actual_DMB_FEB_DAV_Rate") \
  HIST_NAME("ALCT1_vs_ALCT0_KeyWG") \
  HIST_NAME("ALCTTime_Ly%d") \
  HIST_NAME("ALCTTime_Ly%d_Profile") \
  HIST_NAME("ALCT%d_BXN") \
  HIST_NAME("ALCT%d_dTime") \
  HIST_NAME("ALCT%d_dTime_Profile") \
  HIST_NAME("ALCT%d_dTime_vs_KeyWG") \
  HIST_NAME("ALCT%d_KeyWG") \
  HIST_NAME("ALCT%d_Pattern") \
  HIST_NAME("ALCT%d_Pattern_Distr") \
  HIST_NAME("ALCT%d_Quality") \
  HIST_NAME("ALCT%d_Quality_Distr") \
  HIST_NAME("ALCT%d_Quality_Profile") \
  HIST_NAME("ALCT_BXN") \
  HIST_NAME("ALCT_BXN_vs_DMB_BXN") \
  HIST_NAME("ALCT_DMB_BXN_diff") \
  HIST_NAME("ALCT_DMB_L1A_diff") \
  HIST_NAME("ALCT_L1A") \
  HIST_NAME("ALCT_Ly%d_Efficiency") \
  HIST_NAME("ALCT_Ly%d_Rate") \
  HIST_NAME("ALCT_Match_Time") \
  HIST_NAME("ALCT_Number_Efficiency") \
  HIST_NAME("ALCT_Number_Of_Layers_With_Hits") \
  HIST_NAME("ALCT_Number_Of_WireGroups_With_Hits") \
  HIST_NAME("ALCT_Number_Rate") \
  HIST_NAME("ALCT_Word_Count") \
  HIST_NAME("BinCheck_DataFlow_Problems_Frequency") \
  HIST_NAME("BinCheck_DataFlow_Problems_Table") \
  HIST_NAME("BinCheck_ErrorStat_Table") \
  HIST_NAME("BinCheck_Errors_Frequency") \
  HIST_NAME("Chamber_Event_Display_No%d") \
  HIST_NAME("CFEB%d_DMB_L1A_diff") \
  HIST_NAME("CFEB%d_Free_SCA_Cells") \
  HIST_NAME("CFEB%d_L1A_Sync_Time") \
  HIST_NAME("CFEB%d_L1A_Sync_Time_DMB_diff") \
  HIST_NAME("CFEB%d_L1A_Sync_Time_vs_DMB") \
  HIST_NAME("CFEB%d_LCT_PHASE_vs_L1A_PHASE") \
  HIST_NAME("CFEB%d_SCA_Blocks_Locked_by_LCTs") \
  HIST_NAME("CFEB%d_SCA_Blocks_Locked_by_LCTxL1") \
  HIST_NAME("CFEB%d_SCA_Block_Occupancy") \
  HIST_NAME("CFEB_ActiveStrips_Ly%d") \
  HIST_NAME("CFEB_Active_Samples_vs_Strip_Ly%d") \
  HIST_NAME("CFEB_Active_Samples_vs_Strip_Ly%d_Profile") \
  HIST_NAME("AFEB_RawHits_TimeBins") \
  HIST_NAME("CFEB_Clusters_Charge_Ly_%d") \
  HIST_NAME("CFEB_Cluster_Duration_Ly_%d") \
  HIST_NAME("CFEB_Number_of_Clusters_Ly_%d") \
  HIST_NAME("CFEB_Out_Off_Range_Strips_Ly%d") \
  HIST_NAME("CFEB_PedestalRMS_Sample_01_Ly%d") \
  HIST_NAME("CFEB_Pedestal_withEMV_Sample_01_Ly%d") \
  HIST_NAME("CFEB_Pedestal_withRMS_Sample_01_Ly%d") \
  HIST_NAME("CFEB_Pedestal(withEMV)_Sample_01_Ly%d") \
  HIST_NAME("CFEB_Pedestal(withRMS)_Sample_01_Ly%d") \
  HIST_NAME("CFEB_SCA_Cell_Peak_Ly_%d") \
  HIST_NAME("CFEB_Width_of_Clusters_Ly_%d") \
  HIST_NAME("CLCT0_CLCT1_Clssification") \
  HIST_NAME("CLCT0_Clssification") \
  HIST_NAME("CLCT0_KeyDiStrip_vs_ALCT0_KeyWiregroup") \
  HIST_NAME("CLCT1_vs_CLCT0_Key_Strip") \
  HIST_NAME("CLCTTime_Ly%d") \
  HIST_NAME("CLCTTime_Ly%d_Profile") \
  HIST_NAME("CLCT%d_BXN") \
  HIST_NAME("CLCT%d_DiStrip_Pattern") \
  HIST_NAME("CLCT%d_DiStrip_Quality") \
  HIST_NAME("CLCT%d_DiStrip_Quality_Profile") \
  HIST_NAME("CLCT%d_dTime") \
  HIST_NAME("CLCT%d_dTime_Profile") \
  HIST_NAME("CLCT%d_dTime_vs_DiStrip") \
  HIST_NAME("CLCT%d_dTime_vs_Half_Strip") \
  HIST_NAME("CLCT%d_Half_Strip_Pattern") \
  HIST_NAME("CLCT%d_Half_Strip_Quality") \
  HIST_NAME("CLCT%d_Half_Strip_Quality_Distr") \
  HIST_NAME("CLCT%d_Half_Strip_Quality_Profile") \
  HIST_NAME("CLCT%d_KeyDiStrip") \
  HIST_NAME("CLCT%d_KeyHalfStrip") \
  HIST_NAME("CLCT_BXN") \
  HIST_NAME("CLCT_BXN_vs_DMB_BXN") \
  HIST_NAME("CLCT_DMB_BXN_diff") \
  HIST_NAME("CLCT_DMB_L1A_diff") \
  HIST_NAME("CLCT%d_Half_Strip_Pattern_Distr") \
  HIST_NAME("CLCT_L1A") \
  HIST_NAME("CLCT_Ly%d_Efficiency") \
  HIST_NAME("CLCT_Ly%d_Rate") \
  HIST_NAME("CLCT_Number") \
  HIST_NAME("CLCT_Number_Of_HalfStrips_With_Hits") \
  HIST_NAME("CLCT_Number_Of_Layers_With_Hits") \
  HIST_NAME("CLCT_Number_Rate") \
  HIST_NAME("CSC_Efficiency") \
  HIST_NAME("CSC_Rate") \
  HIST_NAME("DMB_BXN_Distrib") \
  HIST_NAME("DMB_BXN_vs_DDU_BXN") \
  HIST_NAME("DMB_CFEB_Active") \
  HIST_NAME("DMB_CFEB_Active_vs_DAV") \
  HIST_NAME("DMB_CFEB_DAV") \
  HIST_NAME("DMB_CFEB_DAV_multiplicity") \
  HIST_NAME("DMB_CFEB_DAV_multiplicity_Unpacking_Inefficiency") \
  HIST_NAME("DMB_CFEB_DAV_Unpacking_Inefficiency") \
  HIST_NAME("DMB_CFEB_MOVLP") \
  HIST_NAME("DMB_CFEB_Sync") \
  HIST_NAME("DMB_DDU_BXN_diff") \
  HIST_NAME("DMB_DDU_L1A_diff") \
  HIST_NAME("DMB_FEB_Combinations_DAV_Efficiency") \
  HIST_NAME("DMB_FEB_Combinations_DAV_Rate") \
  HIST_NAME("DMB_FEB_Combinations_DAV_Unpacking_Inefficiency") \
  HIST_NAME("DMB_FEB_Combinations_Unpacked_vs_DAV") \
  HIST_NAME("DMB_FEB_DAV_Efficiency") \
  HIST_NAME("DMB_FEB_DAV_Rate") \
  HIST_NAME("DMB_FEB_DAV_Unpacking_Inefficiency") \
  HIST_NAME("DMB_FEB_Timeouts") \
  HIST_NAME("DMB_FEB_Unpacked_vs_DAV") \
  HIST_NAME("DMB_FIFO_stats") \
  HIST_NAME("DMB_L1A_Distrib") \
  HIST_NAME("DMB_L1A_vs_ALCT_L1A") \
  HIST_NAME("DMB_L1A_vs_CLCT_L1A") \
  HIST_NAME("DMB_L1A_vs_DDU_L1A") \
  HIST_NAME("DMB_L1_Pipe") \
  HIST_NAME("LCT0_Match_BXN_Difference") \
  HIST_NAME("LCT1_Match_BXN_Difference") \
  HIST_NAME("LCT_Match_Status") \
  HIST_NAME("TMB_ALCT_BXN_diff") \
  HIST_NAME("TMB_ALCT_L1A_diff") \
  HIST_NAME("TMB_BXN_vs_ALCT_BXN") \
  HIST_NAME("TMB_L1A_vs_ALCT_L1A") \
  HIST_NAME("TMB_Word_Count") \
  HIST_NAME("CFEB_Comparators_TimeSamples") \
  HIST_NAME("Buffer_Size") \
  HIST_NAME("BXN") \
  HIST_NAME("CSC_Errors") \
  HIST_NAME("CSC_Errors_Rate") \
  HIST_NAME("CSC_Warnings") \
  HIST_NAME("CSC_Warnings_Rate") \
  HIST_NAME("DMB_Active_Header_Count") \
  HIST_NAME("DMB_Connected_Inputs") \
  HIST_NAME("DMB_Connected_Inputs_Rate") \
  HIST_NAME("DMB_DAV_Header_Count_vs_DMB_Active_Header_Count") \
  HIST_NAME("DMB_DAV_Header_Occupancy") \
  HIST_NAME("DMB_DAV_Header_Occupancy_Rate") \
  HIST_NAME("DMB_unpacked_vs_DAV") \
  HIST_NAME("L1A_Increment") \
  HIST_NAME("Readout_Errors") \
  HIST_NAME("Trailer_ErrorStat_Frequency") \
  HIST_NAME("Trailer_ErrorStat_Rate") \
  HIST_NAME("Trailer_ErrorStat_Table") \
  HIST_NAME("Word_Count") \
  HIST_NAME("All_DDUs_Average_Event_Size") \
  HIST_NAME("All_DDUs_Average_Inputs_with_Data") \
  HIST_NAME("All_DDUs_Average_Live_Inputs") \
  HIST_NAME("All_DDUs_Event_Size") \
  HIST_NAME("All_DDUs_Format_Errors") \
  HIST_NAME("All_DDUs_Inputs_Errors") \
  HIST_NAME("All_DDUs_Inputs_Warnings") \
  HIST_NAME("All_DDUs_Inputs_with_Data") \
  HIST_NAME("All_DDUs_in_Readout") \
  HIST_NAME("All_DDUs_L1A_Increment") \
  HIST_NAME("All_DDUs_Live_Inputs") \
  HIST_NAME("All_DDUs_Trailer_Errors") \
  HIST_NAME("All_Readout_Errors") \
  HIST_NAME("Minus_endcap_AFEB_RawHits_Time") \
  HIST_NAME("Plus_endcap_AFEB_RawHits_Time") \
  HIST_NAME("CSC_AFEB_RawHits_Time_mean") \
  HIST_NAME("CSC_AFEB_RawHits_Time_rms") \
  HIST_NAME("CSC_ALCT0_BXN_mean") \
  HIST_NAME("CSC_ALCT0_BXN_rms") \
  HIST_NAME("Minus_endcap_ALCT0_dTime") \
  HIST_NAME("Plus_endcap_ALCT0_dTime") \
  HIST_NAME("CSC_ALCT0_Quality") \
  HIST_NAME("CSC_ALCT_CLCT_Match_mean") \
  HIST_NAME("CSC_ALCT_CLCT_Match_rms") \
  HIST_NAME("CSC_ALCT_Planes_with_Hits") \
  HIST_NAME("CSC_CLCT0_BXN_mean") \
  HIST_NAME("CSC_CLCT0_BXN_rms") \
  HIST_NAME("CSC_CLCT0_Quality") \
  HIST_NAME("CSC_CLCT_Planes_with_Hits") \
  HIST_NAME("CSC_DMB_input_fifo_full") \
  HIST_NAME("CSC_DMB_input_fifo_full_Fract") \
  HIST_NAME("CSC_DMB_input_timeout") \
  HIST_NAME("CSC_DMB_input_timeout_Fract") \
  HIST_NAME("Minus_endcap_ALCT_CLCT_Match_Time") \
  HIST_NAME("Minus_endcap_CLCT0_dTime") \
  HIST_NAME("Plus_endcap_ALCT_CLCT_Match_Time") \
  HIST_NAME("Plus_endcap_CLCT0_dTime") \
  HIST_NAME("Plus_endcap_CFEB_Comparators_Time") \
  HIST_NAME("Minus_endcap_CFEB_Comparators_Time") \
  HIST_NAME("CSC_CFEB_Comparators_Time_mean") \
  HIST_NAME("CSC_CFEB_Comparators_Time_rms") \
  HIST_NAME("CSC_Format_Errors") \
  HIST_NAME("CSC_Format_Errors_Fract") \
  HIST_NAME("CSC_Format_Warnings") \
  HIST_NAME("CSC_Format_Warnings_Fract") \
  HIST_NAME("CSC_L1A_out_of_sync") \
  HIST_NAME("CSC_L1A_out_of_sync_Fract") \
  HIST_NAME("CSC_Reporting") \
  HIST_NAME("CSC_STATS_cfeb_bwords") \
  HIST_NAME("CSC_STATS_fifofull_err") \
  HIST_NAME("CSC_STATS_format_err") \
  HIST_NAME("CSC_STATS_inputto_err") \
  HIST_NAME("CSC_STATS_l1sync_err") \
  HIST_NAME("CSC_STATS_occupancy") \
  HIST_NAME("CSC_STATS_summary") \
  HIST_NAME("CSC_STATS_wo_alct") \
  HIST_NAME("CSC_STATS_wo_cfeb") \
  HIST_NAME("CSC_STATS_wo_clct") \
  HIST_NAME("CSC_Unpacked") \
  HIST_NAME("CFEB_SCA_CellPeak_Time") \
  HIST_NAME("Plus_endcap_CFEB_SCA_CellPeak_Time") \
  HIST_NAME("Minus_endcap_CFEB_SCA_CellPeak_Time") \
  HIST_NAME("CSC_CFEB_SCA_CellPeak_Time_mean") \
  HIST_NAME("CSC_CFEB_SCA_CellPeak_Time_rms") \
  HIST_NAME("CSC_Unpacked_Fract") \
  HIST_NAME("CSC_Unpacked_with_errors") \
  HIST_NAME("CSC_wo_ALCT") \
  HIST_NAME("CSC_standby") \
  HIST_NAME("CSC_wo_ALCT_Fract") \
  HIST_NAME("CSC_wo_CFEB") \
  HIST_NAME("CSC_wo_CFEB_Fract") \
  HIST_NAME("CSC_wo_CLCT") \
  HIST_NAME("CSC_wo_CLCT_Fract") \
  HIST_NAME("All_DDUs_BXNs") \
  HIST_NAME("DMB_Format_Errors") \
  HIST_NAME("DMB_Format_Errors_Fract") \
  HIST_NAME("DMB_Format_Warnings") \
  HIST_NAME("DMB_Format_Warnings_Fract") \
  HIST_NAME("DMB_input_fifo_full") \
  HIST_NAME("DMB_input_fifo_full_Fract") \
  HIST_NAME("DMB_input_timeout") \
  HIST_NAME("DMB_input_timeout_Fract") \
  HIST_NAME("DMB_L1A_out_of_sync") \
  HIST_NAME("DMB_L1A_out_of_sync_Fract") \
  HIST_NAME("DMB_Reporting") \
  HIST_NAME("DMB_Unpacked") \
  HIST_NAME("DMB_Unpacked_Fract") \
  HIST_NAME("DMB_Unpacked_with_errors") \
  HIST_NAME("DMB_wo_ALCT") \
  HIST_NAME("DMB_wo_ALCT_Fract") \
  HIST_NAME("DMB_wo_CFEB") \
  HIST_NAME("DMB_wo_CFEB_Fract") \
  HIST_NAME("DMB_wo_CLCT") \
  HIST_NAME("DMB_wo_CLCT_Fract") \
  HIST_NAME("FEDEntries") \
  HIST_NAME("FEDFatal") \
  HIST_NAME("FEDFormatFatal") \
  HIST_NAME("FEDNonFatal") \
  HIST_NAME("Physics_EMU") \
  HIST_NAME("Physics_ME1") \
  HIST_NAME("Physics_ME2") \
  HIST_NAME("Physics_ME3") \
  HIST_NAME("Physics_ME4") \
  HIST_NAME("Event_Display_Anode") \
  HIST_NAME("Event_Display_Cathode") \
  HIST_NAME("Event_Display_XY") \
  HIST_NAME("reportSummary") \
  HIST_NAME("CSC_SideMinus") \
  HIST_NAME("CSC_SideMinus_Station01") \
  HIST_NAME("CSC_SideMinus_Station01_Ring01") \
  HIST_NAME("CSC_SideMinus_Station01_Ring02") \
  HIST_NAME("CSC_SideMinus_Station01_Ring03") \
  HIST_NAME("CSC_SideMinus_Station02") \
  HIST_NAME("CSC_SideMinus_Station02_Ring01") \
  HIST_NAME("CSC_SideMinus_Station02_Ring02") \
  HIST_NAME("CSC_SideMinus_Station03") \
  HIST_NAME("CSC_SideMinus_Station03_Ring01") \
  HIST_NAME("CSC_SideMinus_Station03_Ring02") \
  HIST_NAME("CSC_SideMinus_Station04") \
  HIST_NAME("CSC_SideMinus_Station04_Ring01") \
  HIST_NAME("CSC_SideMinus_Station04_Ring02") \
  HIST_NAME("CSC_SidePlus") \
  HIST_NAME("CSC_SidePlus_Station01") \
  HIST_NAME("CSC_SidePlus_Station01_Ring01") \
  HIST_NAME("CSC_SidePlus_Station01_Ring02") \
  HIST_NAME("CSC_SidePlus_Station01_Ring03") \
  HIST_NAME("CSC_SidePlus_Station02") \
  HIST_NAME("CSC_SidePlus_Station02_Ring01") \
  HIST_NAME("CSC_SidePlus_Station02_Ring02") \
  HIST_NAME("CSC_SidePlus_Station03") \
  HIST_NAME("CSC_SidePlus_Station03_Ring01") \
  HIST_NAME("CSC_SidePlus_Station03_Ring02") \
  HIST_NAME("CSC_SidePlus_Station04") \
  HIST_NAME("CSC_SidePlus_Station04_Ring01") \
  HIST_NAME("CSC_SidePlus_Station04_Ring02") \
  HIST_NAME("CertificationSummary") \
  HIST_NAME("CSC_SideMinus") \
  HIST_NAME("CSC_SideMinus_Station01") \
  HIST_NAME("CSC_SideMinus_Station01_Ring01") \
  HIST_NAME("CSC_SideMinus_Station01_Ring02") \
  HIST_NAME("CSC_SideMinus_Station01_Ring03") \
  HIST_NAME("CSC_SideMinus_Station02") \
  HIST_NAME("CSC_SideMinus_Station02_Ring01") \
  HIST_NAME("CSC_SideMinus_Station02_Ring02") \
  HIST_NAME("CSC_SideMinus_Station03") \
  HIST_NAME("CSC_SideMinus_Station03_Ring01") \
  HIST_NAME("CSC_SideMinus_Station03_Ring02") \
  HIST_NAME("CSC_SideMinus_Station04") \
  HIST_NAME("CSC_SideMinus_Station04_Ring01") \
  HIST_NAME("CSC_SideMinus_Station04_Ring02") \
  HIST_NAME("CSC_SidePlus") \
  HIST_NAME("CSC_SidePlus_Station01") \
  HIST_NAME("CSC_SidePlus_Station01_Ring01") \
  HIST_NAME("CSC_SidePlus_Station01_Ring02") \
  HIST_NAME("CSC_SidePlus_Station01_Ring03") \
  HIST_NAME("CSC_SidePlus_Station02") \
  HIST_NAME("CSC_SidePlus_Station02_Ring01") \
  HIST_NAME("CSC_SidePlus_Station02_Ring02") \
  HIST_NAME("CSC_SidePlus_Station03") \
  HIST_NAME("CSC_SidePlus_Station03_Ring01") \
  HIST_NAME("CSC_SidePlus_Station03_Ring02") \
  HIST_NAME("CSC_SidePlus_Station04") \
  HIST_NAME("CSC_SidePlus_Station04_Ring01") \
  HIST_NAME("CSC_SidePlus_Station04_Ring02") \
  HIST_NAME("DAQSummary") \
  HIST_NAME("CSC_SideMinus") \
  HIST_NAME("CSC_SideMinus_Station01") \
  HIST_NAME("CSC_SideMinus_Station01_Ring01") \
  HIST_NAME("CSC_SideMinus_Station01_Ring02") \
  HIST_NAME("CSC_SideMinus_Station01_Ring03") \
  HIST_NAME("CSC_SideMinus_Station02") \
  HIST_NAME("CSC_SideMinus_Station02_Ring01") \
  HIST_NAME("CSC_SideMinus_Station02_Ring02") \
  HIST_NAME("CSC_SideMinus_Station03") \
  HIST_NAME("CSC_SideMinus_Station03_Ring01") \
  HIST_NAME("CSC_SideMinus_Station03_Ring02") \
  HIST_NAME("CSC_SideMinus_Station04") \
  HIST_NAME("CSC_SideMinus_Station04_Ring01") \
  HIST_NAME("CSC_SideMinus_Station04_Ring02") \
  HIST_NAME("CSC_SidePlus") \
  HIST_NAME("CSC_SidePlus_Station01") \
  HIST_NAME("CSC_SidePlus_Station01_Ring01") \
  HIST_NAME("CSC_SidePlus_Station01_Ring02") \
  HIST_NAME("CSC_SidePlus_Station01_Ring03") \
  HIST_NAME("CSC_SidePlus_Station02") \
  HIST_NAME("CSC_SidePlus_Station02_Ring01") \
  HIST_NAME("CSC_SidePlus_Station02_Ring02") \
  HIST_NAME("CSC_SidePlus_Station03") \
  HIST_NAME("CSC_SidePlus_Station03_Ring01") \
  HIST_NAME("CSC_SidePlus_Station03_Ring02") \
  HIST_NAME("CSC_SidePlus_Station04") \
  HIST_NAME("CSC_SidePlus_Station04_Ring01") \
  HIST_NAME("CSC_SidePlus_Station04_Ring02") \
  HIST_NAME("DCSSummary") \
  HIST_NAME("CSC_SideMinus") \
  HIST_NAME("CSC_SideMinus_Station01") \
  HIST_NAME("CSC_SideMinus_Station01_Ring01") \
  HIST_NAME("CSC_SideMinus_Station01_Ring02") \
  HIST_NAME("CSC_SideMinus_Station01_Ring03") \
  HIST_NAME("CSC_SideMinus_Station02") \
  HIST_NAME("CSC_SideMinus_Station02_Ring01") \
  HIST_NAME("CSC_SideMinus_Station02_Ring02") \
  HIST_NAME("CSC_SideMinus_Station03") \
  HIST_NAME("CSC_SideMinus_Station03_Ring01") \
  HIST_NAME("CSC_SideMinus_Station03_Ring02") \
  HIST_NAME("CSC_SideMinus_Station04") \
  HIST_NAME("CSC_SideMinus_Station04_Ring01") \
  HIST_NAME("CSC_SideMinus_Station04_Ring02") \
  HIST_NAME("CSC_SidePlus") \
  HIST_NAME("CSC_SidePlus_Station01") \
  HIST_NAME("CSC_SidePlus_Station01_Ring01") \
  HIST_NAME("CSC_SidePlus_Station01_Ring02") \
  HIST_NAME("CSC_SidePlus_Station01_Ring03") \
  HIST_NAME("CSC_SidePlus_Station02") \
  HIST_NAME("CSC_SidePlus_Station02_Ring01") \
  HIST_NAME("CSC_SidePlus_Station02_Ring02") \
  HIST_NAME("CSC_SidePlus_Station03") \
  HIST_NAME("CSC_SidePlus_Station03_Ring01") \
  HIST_NAME("CSC_SidePlus_Station03_Ring02") \
  HIST_NAME("CSC_SidePlus_Station04") \
  HIST_NAME("CSC_SidePlus_Station04_Ring01") \
  HIST_NAME("CSC_SidePlus_Station04_Ring02") \
  HIST_NAME("FEDBufferSize") \
  HIST_NAME("FED_DDU_L1A_mismatch") \
  HIST_NAME("FEDTotalEventSize") \
  HIST_NAME("FEDTotalCSCs") \
  HIST_NAME("FEDTotalCFEBs") \
  HIST_NAME("FEDTotalALCTs") \
  HIST_NAME("FEDTotalTMBs") \
  HIST_NAME("DCCBufferSize") \
  HIST_NAME("FEDFormat_Errors") \
  HIST_NAME("FED_DDU_L1A_mismatch_fract") \
  HIST_NAME("FED_DDU_L1A_mismatch_with_CSC_data") \
  HIST_NAME("FED_DDU_L1A_mismatch_with_CSC_data_fract") \
  HIST_NAME("FED_DDU_L1A_mismatch_cnt") \
  HIST_NAME("FED_DDU_L1A_mismatch_with_CSC_data_cnt") \
  HIST_NAME("FED_Stats")

#define HIST_ID(a) a,
  enum HistoId {
    HIST_ID_TABLE
  };
#undef HIST_ID

#define HIST_ID(a) # a,
  static const HistoName keys[] = { HIST_ID_TABLE };
#undef HIST_ID

#define HIST_NAME(a) a,
  static const HistoName names[] = { HIST_NAME_TABLE };
#undef HIST_NAME

#define SIZEOF_ARRAY(a) (sizeof a / sizeof (*a))
  static const unsigned int namesSize = SIZEOF_ARRAY(keys);
#undef SIZEOF_ARRAY

#undef HIST_ID_TABLE
#undef HIST_NAME_TABLE
}

#endif
