#ifndef DTtTrigCalibrationTest_H
#define DTtTrigCalibrationTest_H

/** \class DTtTrigCalibrationTest
 * *
 *  DQM Test Client
 *
 *  \author  M. Zanetti CERN
 *   
 */


#include "FWCore/Framework/interface/Frameworkfwd.h"
#include <FWCore/Framework/interface/EDAnalyzer.h>
#include "DataFormats/Common/interface/Handle.h"
#include <FWCore/Framework/interface/ESHandle.h>
#include <FWCore/Framework/interface/Event.h>
#include <FWCore/Framework/interface/MakerMacros.h>
#include "FWCore/ParameterSet/interface/ParameterSet.h"
#include <FWCore/Framework/interface/LuminosityBlock.h>

#include "DQMServices/Core/interface/DQMStore.h"
#include "DQMServices/Core/interface/MonitorElement.h"
#include "FWCore/ServiceRegistry/interface/Service.h"

#include <memory>
#include <iostream>
#include <fstream>
#include <string>
#include <vector>
#include <map>

class DTGeometry;
class DTChamberId;
class DTSuperLayerId;
class DTTtrig;
class DTTimeBoxFitter;

class DTtTrigCalibrationTest: public edm::EDAnalyzer{

public:

  /// Constructor
  DTtTrigCalibrationTest(const edm::ParameterSet& ps);
  
  /// Destructor
  virtual ~DTtTrigCalibrationTest();

protected:

  /// BeginJob
  void beginJob();

  /// BeginRun
  void beginRun(const edm::Run& r, const edm::EventSetup& c);

  /// Analyze
  void analyze(const edm::Event& e, const edm::EventSetup& c);

  /// Endjob
  void endJob();

  /// book the new ME
  void bookHistos(const DTChamberId & ch);

  /// book the summary histograms
  void bookHistos(const DTChamberId & ch, int wh);

  /// Get the ME name
  std::string getMEName(const DTSuperLayerId & slID);

  void beginLuminosityBlock(edm::LuminosityBlock const& lumiSeg, edm::EventSetup const& context) ;

  /// DQM Client Diagnostic
  void endLuminosityBlock(edm::LuminosityBlock const& lumiSeg, edm::EventSetup const& c);




private:

  int nevents;
  unsigned int nLumiSegs;
  int prescaleFactor;
  int run;
  int percentual;

  DQMStore* dbe;

  edm::ParameterSet parameters;
  edm::ESHandle<DTGeometry> muonGeom;
  edm::ESHandle<DTTtrig> tTrigMap;

  DTTimeBoxFitter *theFitter;

  // histograms: < detRawID, Histogram >
  std::map<  uint32_t , MonitorElement* > histos;

  // wheel summary histograms  
  std::map< int, MonitorElement* > wheelHistos;

};

#endif
